/*=====================================================================
  File:      CustAttr.cs

  Summary:   Demonstrates how to create and use custom attributes.

---------------------------------------------------------------------
  This file is part of the Microsoft COM+ 2.0 SDK Code Samples.

  Copyright (C) 2000 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.
=====================================================================*/

// Add the classes in the following namespaces to our namespace
using System;
using System.Reflection;



///////////////////////////////////////////////////////////////////////////////


// This type defines a custom attribute

// The attributes on this attribute describe where the attribute is legal
// and how often the attribute can be applied to a single target
[AttributeUsage(AttributeTargets.Class | AttributeTargets.ClassMembers, 
    AllowMultiple = false)]
public class MyAttribute : System.Attribute {   // Attributes must derive from System.Attribute
    public int y = 111;     // This is a public, named attribute
    public string s;        // This is a public, named attribute
    public int x;           // This is a public, named attribute

    // Since this is the only constructor, every usage of 
    // this attribute requires the 2 positional parameters.
    public MyAttribute(string s, int x) {
        this.s = s;
        this.x = x;
    }

    public static void DisplayAttrInfo(int n, Attribute a) {
        if (a is MyAttribute) {
            // Refer to the one of the custom attributes
            MyAttribute myAttribute = (MyAttribute) a;
            Console.WriteLine("{0}-\"{1}\": {2}", n, a, 
                "X: " + myAttribute.x + ", Y: " + myAttribute.y + ", S: " + myAttribute.s);
        } else {
            Console.WriteLine("{0}-\"{1}\"", n, a);
        }
    }    
}


///////////////////////////////////////////////////////////////////////////////


// This type has our custom attribute applied to it.

// Apply our attribute to our type. the optional named parameter is used.
// NOTE: For convenience, C# allows "Attribute" to be omitted from "MyAttribute"
[My("AttribOnType", 111, y = 222)]
[Obsolete("Ignore this warning -- just testing the Obsolete attribute ")]
class Application {

    // Apply our attribute to this member. the optional named parameter is NOT used.
    [My("AttribOnMethod", 333)]
    public Application() {
        Console.WriteLine("In Application ctor");
    }

    public static void Main() {

        // Get the set of custom attributes associated with the type
        Object[] TypeAttrs = typeof(Application).GetCustomAttributes();
        Console.WriteLine("Number of custom attributes on Application type: " + TypeAttrs.Length);
        for (int n = 0; n < TypeAttrs.Length; n++) {
            MyAttribute.DisplayAttrInfo(n, (Attribute) TypeAttrs[n]);
        }

        // Get the set of methods associated with the type
        MemberInfo[] mi = typeof(Application).FindMembers(
            MemberTypes.Constructor | MemberTypes.Method, 
            BindingFlags.LookupAll | BindingFlags.DeclaredOnly, 
            Type.FilterName, "*");
        Console.WriteLine("Number of methods (includes constructors): " + mi.Length);

        for (int x = 0; x < mi.Length; x++) {
            // Get the set of custom attributes associated with this method
            Object[] MethodAttrs = mi[x].GetCustomAttributes();

            Console.WriteLine("Method name: " + mi[x].Name + "\t("+ MethodAttrs.Length + " attributes)");

            for (int n = 0; n < MethodAttrs.Length; n++) {
                Console.Write("   ");
                MyAttribute.DisplayAttrInfo(n, (Attribute) MethodAttrs[n]);
            }
        }
    }
}


///////////////////////////////// End of File /////////////////////////////////
